/************************************************************
 *  Martin Toepfer     ***     PROJECT : Lingvisto          *
 *  Wissmannstr. 5     ***                                  *
 *  D - 22041 Hamburg  ***     File    : datastructure.c    *
 *  Germany            ***     Date    : 04.04.1997         *
 ************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "datastructure.h"
#include "error.h"

/************************************************************
 *                 some global variables                    *
 ************************************************************/
struct lvo_file head;                   /* head of wordlist */
NODE current;                           /* current wordnode */

/**********************************************************************
 *              basic read/write functions                            *
 **********************************************************************/
/************************************************************
 * set_state : set indicator for unsaved changes            *
 *                                                          *
 * s : one of CHANGED or UNCHANGED                          *
 * !! defined as macro in datastructure.h !!                *
 ************************************************************/
/************************************************************
 * get state : read indicator for unsaved changes           *
 * !! defined as macro in datastructure.h !!                *
 ************************************************************/
/************************************************************
 * set_filename : write filename to 'struct lvo_file head'  *
 *                                                          *
 * filename : filename - string                             *
 ************************************************************/
void set_filename(char *filename) {
  if (filename != NULL) {
    char *f = (char *) malloc(strlen(filename) + 1);

    /*
     * memory available
     */

    if (f != NULL) {
      strcpy(f, filename);
      filename = f;
    }

    /*
     * no memory available
     */

    else {
      lvo_error(NO_MEMORY);
      return;
    }
  }

  /*
   * store old filename
   */

  free(head.filename_bak);
  head.filename_bak = head.filename;
  head.filename = filename;
}

/************************************************************
 * get_filename : read filename from 'struct lvo_file head' *
 * !! defined as macro in datastructure.h !!                *
 ************************************************************/
/************************************************************
 * restore_filename : copy filename backup into filename    *
 ************************************************************/
void restore_filename(void) {
  if (head.filename_bak != NULL) {
    char *fn = (char *) malloc(strlen(head.filename_bak) + 1);
    if (fn != NULL) {
      strcpy(fn, head.filename_bak);
      free(head.filename);
      head.filename = fn;
    }
    else
      lvo_error(NO_MEMORY);
  }
  else
    head.filename = NULL;
}

/************************************************************
 * set_wordlist : set first node of 'head.wordlist'         *
 *                                                          *
 * w : first wordlist - node                                *
 * !! defined as macro in datastructure.h !!                *
 ************************************************************/
/************************************************************
 * get_wordlist : get first node of 'head.wordlist'         *
 * !! defined as macro in datastructure.h !!                *
 ************************************************************/
/************************************************************
 * set_words : write pair of words to 'struct wordnode'     *
 *                                                          *
 * native  : pointer to native language word                *
 * foreign : pointer to foreign language word               *
 * node    : pointer to 'destination node'                  *
 ************************************************************/
void set_words(char *native, char *foreign, NODE node) {
  node->pair.native = native;
  node->pair.foreign = foreign;
}

/**********************************************************************
 *              basic create/destroy functions                        *
 **********************************************************************/
/************************************************************
 * ds_init : initialize datastructure                       *
 ************************************************************/
void init_ds(void) {
  set_filename(NULL);                  /* default : no file */
  head.filename_bak = NULL;
  set_wordlist(NULL);                     /* wordlist empty */
  current = get_wordlist();
  set_state(UNCHANGED);                       /* no changes */
}

/************************************************************
 * n_alloc : allocate memory for new 'struct wordnode'      *
 ************************************************************/
NODE n_alloc(void) {
  return (NODE) malloc(sizeof(struct wordnode));
}

/************************************************************
 * n_free : free memory of 'struct wordnode'                *
 *                                                          *
 * node   : node to free                                    *
 ************************************************************/
void n_free(NODE node)  {
  free(node->pair.native);
  free(node->pair.foreign);
  free(node);
}
